/*
 * InterventionMapBehavior.java
 * Copyright (c) 2011, W4.
 * All rights reserved.
 */

package GestionInterventions.behavior;

import java.net.HttpURLConnection;
import java.net.URL;

import GestionInterventions.src.GESTIONINTERVENTIONS;

import leon.control.LyController;
import leon.control.LyFormController;
import leon.data.LyObject;
import leon.data.LyObjectId;
import leon.info.LyClassInfo;
import leon.view.LyLayout;
import leon.view.LyMap;
import leon.view.LyMapNode;
import leon.view.event.LyDemand;
import eu.w4.ae.app.behavior.LyMapBehavior;
import eu.w4.ae.control.LyMapController;

/**
 * This interface defines the methods to implement in order to modify the
 * behavior of map view controller.
 *
 * @author Lyria S.A.
 * @version $Id: LyMapBehaviorInterface.java 28997 2010-12-23 10:17:15Z bmichel
 *          $
 * @see LyMapController
 */
public class InterventionMapBehavior extends LyMapBehavior {
	// Indicates if the map is initialized or not
	boolean _initialized = false;

	// Indicates whether Google maps are used or not
	boolean _useGoogleMap = false;

	/*
	 * This method is called once to initialize the map: decide whether or not
	 * to use Google Maps
	 */
	@Override
	public void init(LyController controller) {
		super.init(controller);

		// Try establish a URL connection to make sure access to Internet and
		// Google maps is up

		try {
			URL url = new URL(
					"http://maps.googleapis.com/maps/api/js?sensor=false&callback=importgoMapScript");
			HttpURLConnection urlConnection = (HttpURLConnection) url
					.openConnection();
			urlConnection.setConnectTimeout(5000);
			urlConnection.connect();
			_useGoogleMap = true;
		} catch (Exception e) {
			_useGoogleMap = false;
		}

	}

	/**
	 * Complete the map. A geographic layout is used instead of the generic
	 * layout (matrix layout), the viewer is removed, a background is used and
	 * an image is added.
	 *
	 * @param controller
	 *            current mapController instance
	 */
	public void completeMap(LyMapController controller) {
		if (!_initialized) {
			
			IMAGE_HEIGHT = 32;
			// Create a geographic layout
			LyMap     map = controller.getMapComponent();
			LyLayout  layout = new LyLayout();

			map.setStyle(_useGoogleMap ? LyMap.Style.TERRAIN : LyMap.Style.STANDARD);
			layout.setStyle(LyLayout.Style.GEOGRAPHIC);
			map.setLayout(layout);

			if (!_useGoogleMap)
			{
				LyMapNode france = new LyMapNode();

				france.setId("france_node");
				france.setImage("france256.gif");
				france.setStyle(LyMapNode.Style.IMAGE);
				france.setViewerStyle(LyMapNode.Style.IMAGE);
				france.setLayer(0);
				france.setResizable(true);
				france.setSensitive(false);
				map.addMapNode(france);
			}
			else
			{
				map.setLongitude("2.704004");
				map.setLatitude("46.348813");
			}

			map.setViewer(true);
			map.setZoom(true);
			map.setTranslation(true);
				map.setZoomFactor(_useGoogleMap ? 6 : 1);
				map.setWidth(900);
				map.setHeight(575);

			_initialized = true;
		}
	}

	/*
	 * This method is called once for each object to display on the map:
	 */
	@Override
	public void updateNode(LyObject nodeObject, LyMapNode mapNode,
			LyMapController controller) {
		// Update the node in the generic way
		super.updateNode(nodeObject, mapNode, controller);

		// Retrieve the longitude/latitude for the object
		Object latitude = null;
		Object longitude = null;

		if (controller.getParent().getParent() instanceof LyFormController) {
			// intervention form controller
			LyFormController taskController = (LyFormController) controller.getParent().getParent();
			LyClassInfo taskCls = (taskController).getClassInfo();
			LyFormController interventionController = (LyFormController) (taskController).getAreaController(taskCls.getFieldInfo(GESTIONINTERVENTIONS.FLD_ACT_INTERVENTION_DEMANDEE_FIELD_6));
			LyObject intervention = interventionController.getObject();

			// specific update for the object of the current view
			if (interventionController != null && nodeObject.getId().equals(intervention.getId())) {
				// address form controller
				LyClassInfo interventionCls = (interventionController).getClassInfo();
				LyFormController addressController = (LyFormController) (interventionController).getAreaController(interventionCls.getFieldInfo(GESTIONINTERVENTIONS.FLD_INTERVENTION_ADDRESS));

				// retreive the coordinate from the form controller
				latitude = addressController.getFieldValue(GESTIONINTERVENTIONS.FLD_ADDRESS_LATITUDE).getValue();
				longitude = addressController.getFieldValue(GESTIONINTERVENTIONS.FLD_ADDRESS_LONGITUDE).getValue();
			}

		}

		if (latitude == null || longitude == null) {
			// 1 ) retreive the intervention address

			LyObjectId addressId = (LyObjectId) nodeObject.getValue(GESTIONINTERVENTIONS.FLD_INTERVENTION_ADDRESS);

			if (addressId != null) {
				LyObject address = addressId.getLogicalObject();
				// 1 ) retreive longitude/latitude from the address
				if (address != null) {
				  latitude = address.getValue(GESTIONINTERVENTIONS.FLD_ADDRESS_LATITUDE);
				  longitude = address.getValue(GESTIONINTERVENTIONS.FLD_ADDRESS_LONGITUDE);
				}
			}
		}
//		// default localisation : france
//		if (x == null || y == null) {
//			x = "2.704004";
//			y = "46.348813";
//		}

		// Set the node position and image with
		if (latitude != null && latitude.toString().length() > 0 && longitude != null
				&& longitude.toString().length() > 0) {

			mapNode.setX(Float.parseFloat(longitude.toString()));
			mapNode.setY(Float.parseFloat(latitude.toString()));
			mapNode.setViewerStyle(LyMapNode.Style.IMAGE);
		}
	}

	@Override
	public boolean handleDemand(LyController controller, LyDemand demand) {
		String source = demand.getSource();
		// when the close button is selected, the window is closed.
		if (source != null && source.equals(LyFormController.DEMAND_CLOSE)) {
			controller.free();
			return true;
		}
		return super.handleDemand(controller, demand);
	}

}
